<?php

use function apexl\io\env;

return [
    "environment" => env('APP_ENVIRONMENT', 'production'),
    "name" => env('APP_NAME', 'IO Application'),
    "email_address" => env('APP_EMAIL_ADDRESS', 'no-reply@localhost'),
];