<?php

declare(strict_types=1);

namespace apexl\Io\interfaces;

interface HasGeneralSearchInterface
{
    public function generalSearchFields(): array;

    public function relation(string $relation): ?RelationshipInterface;
}
