<?php

declare(strict_types=1);

namespace apexl\Io\interfaces;

interface LazyLoadAllowerInterface
{
    public function isAllowed(): bool;

    /**
     * @return ?bool - The previous override value
     */
    public function override(bool $override): ?bool;
}